# pqfax v0.1: graphical fax viewer
#
# Copyright 2002 Hans-Peter Jansen <hpj@urpla.net>
#
# This program is placed under the GNU General Public License V.2

import sys
import math
from qt import *
from qth import QTIFFHandler
from pqfaxdial import pqfaxDial

true = 1
false = 0

#FaxDir = "/home/hp/CVS/qfaxreader.hp/test"
FaxDir = "/var/spool/fax/recvq"

class faxView(QScrollView):
    def __init__(self, tiffHandler, parent, name = None):
        QScrollView.__init__(self, parent, name)
        self.tiffHandler = tiffHandler
        self.viewport().setEraseColor(QColor(255, 255, 255))
        self.viewport().setMaximumSize(0, 0)
        self.resizeContents(0, 0)
        self.setResizePolicy(QScrollView.AutoOneFit)

    def drawContentsOffset(self, p, ox, oy, cx, cy, cw, ch):
        if not self.tiffHandler.pages() or not self.tiffHandler.currentPage():
            return
        #print "drawContentsOffset", ox, oy, cx, cy, cw, ch
        p.setClipRect(cx - ox, cy - oy, cw, ch)
        rowheight = self.tiffHandler.currentPage().height()
        toprow = cy / rowheight
        bottomrow = (cy + ch + rowheight - 1) / rowheight
        colwidth = self.tiffHandler.currentPage().width()
        leftcol = cx / colwidth
        rightcol = (cx + cw + colwidth - 1) / colwidth
        for r in range(toprow, bottomrow):
            py = r * rowheight
            for c in range(leftcol, rightcol):
                px = c * colwidth
                p.drawPixmap(px - ox, py - oy, self.tiffHandler.currentPage())

    def updateContent(self):
        if self.tiffHandler.currentPage():
            #print "updateContent"
            self.viewport().setMaximumSize(self.tiffHandler.currentPage().width(),
                                           self.tiffHandler.currentPage().height())
            self.resizeContents(self.tiffHandler.currentPage().width(),
                                self.tiffHandler.currentPage().height())
            self.viewport().repaint()
        else:
            self.viewport().setMaximumSize(0, 0)
            self.resizeContents(0, 0)
            self.viewport().repaint()

class mainWindow(pqfaxDial):
    def __init__(self, app, parent = None, name = None, fl = 0):
        self.app = app
        self.fv = None
        self.zoomLevel = [0.25, 0.33, 0.5, 0.66, 0.75, 0.9, 1.0, 1.2, 1.4, 1.6, 1.8, 2.0]
        pqfaxDial.__init__(self, parent, name, fl)

    def accept(self):
        print "accept"
        self.closeEvent()
        QDialog.accept(self)

    def closeEvent(self, e = None):
        print "close"
        sys.exit(0)

    def polish(self):
        #print "polish"
        if not self.__dict__.has_key("tiffHandler"):
            try:
                self.tiffHandler = QTIFFHandler(self.statusMsg, self, "tiffHandler", 0.5, true)
            except:
                QMessageBox.warning(self, "pqfax Viewer",
                    ">qth< not found. Please make sure, this\n"
                    "C++ sip extension module is properly installed\n",
                    QMessageBox.Abort | QMessageBox.Escape)
                sys.exit(1)

            self.fv = faxView(self.tiffHandler, self.faxFrame, "faxviewer")
            for v in self.zoomLevel:
                self.zoomCombo.insertItem("%d%%" % int(v * 100))
            self.updateGUI()

            self.connect(self.tiffHandler, SIGNAL("update()"), self.updateGUI)
            self.connect(self.tiffHandler, SIGNAL("update()"), self.fv.updateContent)

            # signals & slots
            self.connect(self.openFaxFile, SIGNAL("clicked()"), self.openFax)
            self.connect(self.pageDownButton, SIGNAL("clicked()"), self.setPageDown)
            self.connect(self.pageCombo, SIGNAL("activated(int)"), self.setPageIdx)
            self.connect(self.pageUpButton, SIGNAL("clicked()"), self.setPageUp)
            self.connect(self.zoomInButton, SIGNAL("clicked()"), self.zoomIn)
            self.connect(self.zoomCombo, SIGNAL("activated(int)"), self.setZoom)
            self.connect(self.zoomOutButton, SIGNAL("clicked()"), self.zoomOut)
            self.connect(self.rotateLeftButton, SIGNAL("clicked()"), self.rotateLeft)
            self.connect(self.rotateRightButton, SIGNAL("clicked()"), self.rotateRight)
            self.connect(self.flipButton, SIGNAL("clicked()"), self.flip)
            self.connect(self.smoothScaleButton, SIGNAL("toggled(bool)"), self.toggleSmoothScale),
            self.connect(self.quitDialog, SIGNAL("clicked()"), self.closeEvent)

        QDialog.polish(self)
        #self.openFax()

    def updateGUI(self):
        if self.tiffHandler.smoothScale():
            self.smoothScaleButton.setOn(true)
        else:
            self.smoothScaleButton.setOn(false)
        idx = 0
        try:
            idx = self.zoomLevel.index(math.floor(self.tiffHandler.zoom()*100)/100)
        except ValueError:
            try:
                idx = self.zoomLevel.index(math.ceil(self.tiffHandler.zoom()*100)/100)
            except ValueError:
                print "float fuckup"
        if idx == 0:
            self.zoomInButton.setEnabled(false)
        else:
            self.zoomInButton.setEnabled(true)
        if idx == len(self.zoomLevel) - 1:
            self.zoomOutButton.setEnabled(false)
        else:
            self.zoomOutButton.setEnabled(true)
        self.zoomCombo.setCurrentItem(idx)
        self.pageCombo.clear()
        pages = self.tiffHandler.pages()
        if not pages:
            self.pageDownButton.setEnabled(false)
            self.pageCombo.setEnabled(false)
            self.pageUpButton.setEnabled(false)
            self.rotateLeftButton.setEnabled(false)
            self.rotateRightButton.setEnabled(false)
            self.flipButton.setEnabled(false)
        else:
            page = self.tiffHandler.page()
            for p in range(1, pages + 1):
                self.pageCombo.insertItem("%d" % p)
            self.pageCombo.setCurrentItem(page - 1)
            if pages > 1:
                self.pageCombo.setEnabled(true)
                if page == 1:
                    self.pageDownButton.setEnabled(false)
                else:
                    self.pageDownButton.setEnabled(true)
                if page == pages:
                    self.pageUpButton.setEnabled(false)
                else:
                    self.pageUpButton.setEnabled(true)
            else:
                self.pageDownButton.setEnabled(false)
                self.pageCombo.setEnabled(false)
                self.pageUpButton.setEnabled(false)
            self.rotateLeftButton.setEnabled(true)
            self.rotateRightButton.setEnabled(true)
            self.flipButton.setEnabled(true)

    def openFax(self):
        print "open fax"
        file = QFileDialog.getOpenFileName(FaxDir,
                "Tiff Files (*.tif *.tiff *.TIF *.TIFF)", self)
        if file.isNull():
            return
        self.tiffHandler.load(file.latin1())

    def setPageDown(self):
        page = self.tiffHandler.page() - 1
        if page >= 1:
            self.setPage(page)

    def setPageIdx(self, idx):
        self.setPage(idx + 1)

    def setPage(self, page):
        self.pageDownButton.setEnabled(false)
        self.pageCombo.setEnabled(false)
        self.pageUpButton.setEnabled(false)
        self.tiffHandler.setPage(page)

    def setPageUp(self):
        page = self.tiffHandler.page() + 1
        if page <= self.tiffHandler.pages():
            self.setPage(page)

    def zoomIn(self):
        idx = self.zoomCombo.currentItem() - 1
        if idx >= 0:
            self.tiffHandler.scale(self.zoomLevel[idx])
            #self.zoomCombo.setCurrentItem(idx)
        #self.updateGUI()

    def setZoom(self, idx):
        self.tiffHandler.scale(self.zoomLevel[idx])
        self.updateGUI()

    def zoomOut(self):
        idx = self.zoomCombo.currentItem() + 1
        if idx < len(self.zoomLevel):
            self.tiffHandler.scale(self.zoomLevel[idx])
            #self.zoomCombo.setCurrentItem(idx)
        #self.updateGUI()

    def rotateLeft(self):
        self.tiffHandler.rotate(-90.0)

    def rotateRight(self):
        self.tiffHandler.rotate(90.0)

    def flip(self):
        self.tiffHandler.rotate(180.0)

    def toggleSmoothScale(self, state):
        self.tiffHandler.setSmoothScale(state)
        if self.tiffHandler.pages():
            self.tiffHandler.setPage(self.tiffHandler.page())

    def resizeEvent(self, e = None):
        if self.fv:
            self.fv.resize(self.faxFrame.size())
