#!/usr/bin/env python

# vim: expandtab

import os, re, sys
from os.path import basename, exists, join
from glob import glob
from types import *

from distutils.ccompiler import new_compiler
from distutils.command.build_ext import build_ext
from distutils.command.install_data import install_data
from distutils.core import Extension
from distutils.errors import DistutilsFileError
from distutils.spawn import find_executable
from distutils.sysconfig import get_config_vars, get_python_inc, get_python_lib
from distutils.util import byte_compile

import qt

#------------------------------------------------------------------------------
# The sip-generated output is fixed up by means of the following functions
#
def replacer(matchObj):
    if matchObj.group(2) == '*args':
        return (
            "%(head)s\n"
            "%(tabs)s\tfor key in kwargs.keys():\n"
            "%(tabs)s\t\tmethod = getattr(self, key)\n" 
            "%(tabs)s\t\tif type(kwargs[key]) == types.TupleType:\n" 
            "%(tabs)s\t\t\tapply(method,kwargs[key])\n"
            "%(tabs)s\t\telse:\n"
            "%(tabs)s\t\t\tmethod(kwargs[key])\n" %
            { 'head' : re.sub('\*args', '*args,**kwargs', matchObj.group(0)),
              'tabs' : matchObj.group(1) } )


def keywords(name, text):
    library = os.path.basename(name).split('.')[0]
    initRE = re.compile(r'(\t+)def __init__\(self,(\*args)\):\n'
                        '(\t+)lib%sc\.sipCallCtor\(\d+,self,args\)' % library)
    text = re.sub("\nimport lib%s" % library,
                  "\nimport types\nimport lib%s" % library, text)
    text = re.sub(initRE, replacer, text)
    return text


# Windows and Unix behave differently: Unix does not need os.remove()
def backup(source, extension):
    target = source + extension
    if exists(target):
        os.remove(target)
    os.rename(source, target)


def fixup(dir, name):
    return
    print '...fixing up sip output by means of fixup(dir, name)'
    
    if 0:
        py_module = join(dir, '%s.py' % name)
        print '...fixup import lib%sc to lib%scmodule' % (name, name)
        file = open(py_module, 'r')
        text = file.read()
        file.close()
        text = text.replace('import lib%sc' % name,
                            'import lib%smodule' % name)
        backup(py_module, '.bak')
        file = open(py_module, 'w')
        file.write(text)
        file.close()

    py_wrapper = join(dir, '%s.py' % name)
    print '...fixing the keyword arguments in the Python constructors'
    file = open(py_wrapper, 'r')
    text = file.read()
    file.close()
    text = keywords(name, text)
    backup(py_wrapper, '.keywords')
    file = open(py_wrapper, 'w')
    file.write(text)
    file.close()

#------------------------------------------------------------------------------
# on Windows we glob for a sensible library name
# on Linux/Unix we find the precise Qt library name by peeking in libqtcmodule
#
def find_qt_library_name(qt_dir):
    if os.name == 'nt':
        # assume multi-threading
        template = "qt-mt%s*.lib"
        tag = qt.qVersion().replace('.', '')
        try:
            name = glob(join(qt_dir, 'lib', template % tag))[0]
        except IndexError:
            raise SystemExit, \
                  ("Failed to find a library matching '%s'" %
                   os.join(qt_dir, 'lib', (template % tag)))
        # zap the trailing '.lib'
        return basename(name)[:-4]
    elif os.name == 'posix':
        module = None
        # execute yourself to find your PyQt extension module
        _, _, stderr = os.popen3("%s -v -c 'import libsip, libqtc'" %
                                 sys.executable)
        for line in stderr.readlines():
            words = line.split()
            if words[0] == 'import' and words[1] == 'libqtc':
                module = words[-1].strip()
                break
        if not module:
            raise SystemExit, \
                  "Failed to find Qt's library name in the PyQt extension"
        # execute ldd to find the name of the Qt library
        _, stdout, _ = os.popen3("ldd %s" % module)
        for line in stdout.readlines():
            name = line.split()[0].split('.')[0][3:]
            if name[:2] == 'qt':
                return name
        raise SystemExit, \
              "Failed to find Qt's library name in the PyQt extension"
    else:
        raise SystemExit, \
              "PyQth is not compatible with Qt on '%s'" % os.name

#------------------------------------------------------------------------------
# - Linux/Unix we transform the UnixCCompiler into a UnixC++Compiler
# - Windows we select between "windows" and "console" applications
#
def customize_compiler(compiler, windows_app = "windows"):
    """Hide distutils.sysconfig.customize(compiler).
    """
    if compiler.compiler_type == "unix":
        # ./configure --with-cxx=c++ sets CXX=c++ in config/Makefile
        (cc, cxx, opt, ccshared, ldshared, so_ext) = \
            get_config_vars('CC', 'CXX', 'OPT', 'CCSHARED', 'LDSHARED', 'SO')
        # fall back on g++
        cxx = cxx or 'g++'
        if not find_executable(cxx):
            raise SystemExit, "Fails to find C++ compiler '%s' in path" % cxx
        cxx_cmd = cxx + ' -Wall ' + opt + ' -fno-exceptions'
        compiler.set_executables(preprocessor=cxx + " -E",
                                 compiler=cxx_cmd,
                                 compiler_so=cxx_cmd + ' ' + ccshared,
                                 linker_so=ldshared.replace(cc, cxx),
                                 linker_exe=cxx)
        compiler.shared_lib_extension = so_ext
    if compiler.compiler_type == "msvc":
        compiler.compile_options = [ '/nologo', '/W3', '/MT', '/O1' ]
        if windows_app == "windows":
            compiler.ldflags_shared = ['/DLL', '/nologo', '/SUBSYSTEM:windows']
        elif windows_app == "console":
            compiler.ldflags_shared = ['/DLL', '/nologo', '/SUBSYSTEM:console']
        else:
            raise SystemExit, "Unknown MSVC application type"


#------------------------------------------------------------------------------
# sip needs to know the Qt version and the Window system
#
def timeline_platform_options(qt_version):
    message = "PyQth is not compatible with QT_VERSION '%s'"
    if qt_version < 200:
        raise SystemExit, message % qt_version
    elif qt_version < 210:
        timeline = 'Qt_2_00' 
    elif qt_version < 220:
        timeline = 'Qt_2_1_0'
    elif qt_version < 230:
        timeline = 'Qt_2_2_0'
    elif qt_version == 230:
        timeline = 'Qt_2_3_0'
    elif qt_version == 231:
        timeline = 'Qt_2_3_1'
    elif qt_version == 300:
        timeline = 'Qt_3_0_0'
    elif qt_version == 301:
        timeline = 'Qt_3_0_1'
    elif qt_version == 302 or qt_version == 303:
        timeline = 'Qt_3_0_2'
    elif qt_version == 304:
        timeline = 'Qt_3_0_4'
    elif qt_version == int("0x030005", 16):
        timeline = 'Qt_3_0_5'
    elif qt_version == int("0x030100", 16):
        timeline = 'Qt_3_1_0'
    else: # FIXME: Qt-4.0.0
        raise SystemExit, \
              "PyQth is not compatible with QT_VERSION '%s'" % qt_version
    if os.name == 'posix':
        platform = 'WS_X11'
    elif os.name == 'nt':
        platform = 'WS_WIN'
    else:
        raise SystemExit, \
              "PyQth is not compatible with Qt on '%s'" % os.name
    return '-t %s -t %s' % (timeline, platform)


#------------------------------------------------------------------------------
#
#
class PyQtExtension(Extension):

    def __init__(self, name, sources,
                 sip_spec_dirs=None,    # PyQtExtension
                 moc_headers=None,      # PyQtExtension
                 include_dirs=None,
                 define_macros=None,
                 undef_macros=None,
                 library_dirs=None,
                 libraries=None,
                 runtime_library_dirs=None,
                 extra_objects=None,
                 extra_compile_args=None,
                 extra_link_args=None):

        assert isinstance(name, StringType), "'name' must be a string"
        assert (sources == []) or (isinstance(sources, ListType) and
                map(isinstance, sources, [StringType]*len(sources))), \
                "'sources' must be an empty list or a list of strings"

        self.sip_spec_dirs = sip_spec_dirs or []
        self.moc_headers = moc_headers or []

        self.last = name.split('.')[-1]

        self.sip_file = join('sip', '%s.sip' % self.last)
        self.sip_wrapper = join(self.last, '%s.py' % self.last)

        # check if all your hand written C++ sources exist
        for source in sources:
            if not exists(source):
                raise DistutilsFileError, "file '%s' does not exist" % source 
        sources = sources
        # the main sip-generated source file, find the rest after running sip
        sources.append(join(self.last, '%scmodule.cpp' % self.last))
        # handle the header files to be processed by moc, implies more sources
        self.moc_iofiles = []
        for header in self.moc_headers:
            if not exists(source):
                raise DistutilsFileError, "file '%s' does not exist" % header 
            head, tail = os.path.split(header)
            (root, _) = os.path.splitext(tail)
            target = join(head, 'moc_%s.cpp' % root)
            self.moc_iofiles.append((header, target))
            if target not in sources:
                sources.append(target)

        if os.name == 'nt':
            self.name = 'qth.lib%sc' % self.last
        else:
            #self.name = 'qth.lib%scmodule' % self.last
            self.name = 'qth.lib%sc' % self.last
        # now call the base class constructor with the sources we've found
        Extension.__init__(self,
                           self.name,
                           sources,
                           include_dirs,
                           define_macros,
                           undef_macros,
                           library_dirs,
                           libraries,
                           runtime_library_dirs,
                           extra_objects,
                           extra_compile_args,
                           extra_link_args)
        self.include_dirs.append(join(os.getcwd(), self.last))


#------------------------------------------------------------------------------
#
#
class build_ext_pyqt(build_ext):
    description = "build 'PyQt' extensions, using 'sip'"

    user_options = [
        ('qt-dir=', None,
         "Qt's root directory, e.g. /usr/lib/qt2"),
        ('qt-bindir=', None,
         "Qt's binary directory for moc & friends, e.g. /usr/lib/qt2/bin"),
        ('qt-incdir', None,
         "Qt's include directory, e.g. /usr/lib/qt2/include"),
        ('moc-program=', None,
         "full path of moc"),
        ('sip-program=', None,
         "full path of sip"),
        ('sip-only', None,
         "run sip to build Python and C++ sources from *.sip files")
        ]

    def initialize_options(self):
        self.qt_dir = None
        self.qt_bindir = None
        self.qt_incdir = None
        self.qt_libdir = None
        self.moc_program = None
        self.sip_program = None
        self.sip_only = 0
        build_ext.initialize_options(self)

    def finalize_options(self):
        build_ext.finalize_options(self)

        # try to get our options
        if not self.qt_dir:
            if os.environ.has_key("QTDIR"):
                self.qt_dir = os.environ["QTDIR"]
        if self.qt_dir:
            if not self.qt_bindir:
                self.qt_bindir = join(self.qt_dir, 'bin')
            if not self.qt_incdir:
                self.qt_incdir = join(self.qt_dir, 'include')
            if not self.qt_libdir:
                self.qt_libdir = join(self.qt_dir, 'lib')
            if not self.moc_program:
                self.moc_program = find_executable('moc', self.qt_bindir)

        if not self.moc_program:
            self.moc_program = find_executable('moc')
        if not self.sip_program:
            self.sip_program = find_executable('sip')

        # BOMB out, if self.qt_incdir is WRONG, get QT_VERSION from 'qglobal.h'
        if not self.qt_incdir:
            raise SystemExit, \
                  "Edit 'setup.py': 'qt_incdir=%s' fails." % self.qt_incdir
        else:   
            filename = join(self.qt_incdir, 'qglobal.h')
            try:
                file = open(filename)
                text = file.read()
                file.close()
                mo = re.compile(
                    r'^#define\s+QT_VERSION\s+(\d\d\d)|(0x\d\d\d\d\d\d)',
                    re.M).search(text, 1)
                if not mo:
                    raise IndexError
                if mo.group(1): 
                    self.qt_version = int(mo.group(1))
                elif mo.group(2):
                    self.qt_version = int(mo.group(2), 16)
                else:
                    raise IndexError
            except IOError:
                raise SystemExit, \
                      "Edit 'setup.py': %s does not exist." % filename
            except IndexError:
                raise SystemExit, \
                      "Edit 'setup.py': %s does not '#define QT_VERSION'." % \
                      filename

        # BOMB out, if self.sip_program is not OK
        if not self.sip_program:
            raise SystemExit, ("Edit 'setup.py': 'sip_program=%s' fails." %
                               self.sip_program)
        sip_version = os.popen('%s -V' % self.sip_program).read().split()[0]
        pyqt_version = qt.PYQT_VERSION

        # finalize
        self.sip_timeline_platform = timeline_platform_options(self.qt_version)
        if self.qt_incdir:
            self.include_dirs.append(self.qt_incdir)
        self.library_dirs.append(get_python_lib())
        if self.qt_libdir:
            self.library_dirs.append(self.qt_libdir) 
        self.define = self.define or []
        self.define.append(('SIP_MAKE_MODULE_DLL', None))
        self.define.append(('QT_NODEBUG', None))
        qt_library_name = find_qt_library_name(self.qt_dir)
        self.libraries = self.libraries + [qt_library_name]
        if os.name == 'nt':
            self.define.append(('QT_DLL', None))
            # assume multi-threading on windows
            self.define.append(('QT_THREAD_SUPPORT', None))
            # hack, hack, ...
            if qt_library_name == 'qt-mt230nc':
                extra_version = '-Qt-NC'
            else:
                extra_version = '-Qt-' + qt_library_name[5:]
            self.distribution.metadata.version = \
                 self.distribution.metadata.version + extra_version
        else:
            if -1 != qt_library_name.find("mt"):
                self.define.append(('_REENTRANT', None))
                self.define.append(('QT_THREAD_SUPPORT', None))
                
    def run_sip(self, extension):
        # run sip with appropriate arguments
        includes = ['-I', 'sip']
        for dir in extension.sip_spec_dirs:
            includes.append('-I')
            # hack: C code does not understand Windows path separators
            includes.append(dir.replace('\\', '/'))
        name = extension.last
        cmd = [self.sip_program] + includes + ['-c', name] + \
              self.sip_timeline_platform.split() + \
              [extension.sip_file.replace('\\', '/')]
        # do we need features?      ['-z', "features"] + \
        self.spawn(cmd)
        # postprocess the sip output
        fixup(name, name)
        # python package
        self.copy_file(join(name, name + '.py'), '%s/__init__.py' %
                       '.'.join(extension.name.split('.')[0:-1]))
        
    def run_moc(self, extension):
        # run moc, we get along without the preprocessor step!
        name = extension.last
        sipProxy_h = join(name, 'sip%sProxy%s.h' % (name, name))
        sipProxy_moc = join(name, 'moc_sip%sProxy%s.cpp' % (name, name))
        self.make_file([sipProxy_h], sipProxy_moc, self.spawn,
                       ([self.moc_program, sipProxy_h, '-o', sipProxy_moc],))
        self.announce('"%s" has made "%s"' % (self.moc_program, sipProxy_moc))
        for ifile, ofile in extension.moc_iofiles:
            self.make_file([ifile], ofile, self.spawn,
                           ([self.moc_program, ifile, '-o', ofile],))


    def run(self):
        for extension in self.extensions:
            assert isinstance(extension, PyQtExtension)
            # sip sources are 'extension.sip_filename' and files included by it
            extension.sip_files = [ extension.sip_file ]
            file = open(extension.sip_file, 'r')
            IncludeRE = re.compile(r'^%Include (.*\.sip)')
            for line in file.readlines():
                m = re.match(IncludeRE, line)
                if m:
                    inclusion = join('sip', m.group(1))
                    if exists(inclusion):
                        extension.sip_files.append(inclusion)
                    else:
                        raise DistutilsFileError, \
                              "file '%s' does not exist" % inclusion
            file.close()
            # run sip and post-process, if a sip file is newer than the wrapper
            self.make_file(extension.sip_files,
                           extension.sip_wrapper,
                           self.run_sip, (extension,))
            # run moc
            self.run_moc(extension)
            # append the sip- and moc- generated *.cpp to the sources
            name = extension.last
            sources = glob(join(name, "*sip%s*.cpp" % name))
            for source in sources:
                extension.sources.append(source)

        if self.sip_only:
            return 1

        # the compiler is needed BEFORE building the extensions
        self.compiler = new_compiler(verbose=self.verbose,
                                     dry_run=self.dry_run,
                                     force=self.force)
        customize_compiler(self.compiler)
            
        if self.include_dirs is not None:
            self.compiler.set_include_dirs(self.include_dirs)
        if self.define is not None:
            # 'define' option is a list of (name,value) tuples
            for (name, value) in self.define:
                self.compiler.define_macro(name, value)
        if self.undef is not None:
            for macro in self.undef:
                self.compiler.undefine_macro(macro)
        if self.libraries is not None:
            self.compiler.set_libraries(self.libraries)
        if self.library_dirs is not None:
            self.compiler.set_library_dirs(self.library_dirs)
        if self.rpath is not None:
            self.compiler.set_runtime_library_dirs(self.rpath)
        if self.link_objects is not None:
            self.compiler.set_link_objects(self.link_objects)
        self.compiler.set_include_dirs(self.include_dirs)
        self.compiler.set_libraries(self.libraries)

        # add the PyQt/sip libraries now
        if os.name == 'posix':
            self.compiler.add_library('qtcmodule')
            self.compiler.add_library('sip')
        elif os.name == 'nt':
            self.compiler.add_library('libqtc')
            self.compiler.add_library('libsip')

        self.build_extensions()

        for extension in self.extensions:
            name = extension.last
            package = '.'.join(extension.name.split('.')[0:-1])
            build_py = self.get_finalized_command('build_py')
            package_dir = build_py.get_package_dir(package)
            wrapper = join(name, '%s.py' % name)
            self.copy_file(wrapper, join(package_dir, '__init__.py'))


#------------------------------------------------------------------------------
#
#
class install_data_pyqt(install_data):   
    def finalize_options (self):
        self.set_undefined_options('install',
                                   ('install_lib', 'install_dir'),
                                   ('root', 'root'),
                                   ('force', 'force'))

