#!/usr/bin/env python

import os

# EDIT the Windows or Linux/Unix section to reflect your system:
# qt_dir -- must contain your Qt distribution (headers, binaries, libraries)
# sip_program -- must be the full path to the sip program
# sip_spec_dirs -- must contain the files 'qtmod.sip' & friends

if os.name == 'nt':
    # Windows-95, -98, ...
    # My PyQwt build procedure is (see HTML documentation):
    # (1) build and install sip from source
    # (2) build and install PyQt manually from source
    # (3) build PyQwt from source
    qt_dir = 'C:\qt'
    sip_program = 'sip.exe' # by default sip is installed in C:\Python2.2.
    sip_spec_dirs = ['C:\MyStuff\PyQt-3.3.2\sip']
else:
    # Linux/Unix
    qt_dir = '/usr/lib/qt3'
    sip_program = '/usr/bin/sip'
    sip_spec_dirs = ['/usr/share/sip/qt']

#
# IF YOU NEED TO EDIT BELOW THIS LINE -- IT IS A BUG
#

#
# CONFIGURATION INFORMATION
#

srcdir = 'src'

from qt import qVersion, PYQT_VERSION

version = '0.1.1'

QT_VERSION = qVersion()

#
# CHECK FOR A RECENT PYTHON
#

import sys
if not hasattr(sys, 'version_info') or sys.version_info < (2,0,0, 'alpha', 0):
    raise SystemExit, 'Python-2.0 or later required to build PyQwt.'

#
# REAL WORK
#

# pyqt_support.py uses this (replaces command line switches)
file = open('setup.cfg', 'w')
file.write('[build_ext]\n')
file.write('qt_dir=%s\n' % qt_dir)
file.write('sip_program=%s\n' % sip_program)
file.close()

# postpone costly module imports as much possible
from distutils.core import setup
from distutils.file_util import copy_file
from distutils.sysconfig import get_python_inc
from glob import glob
from os import mkdir
from os.path import exists, join

from pyqt_support import PyQtExtension, build_ext_pyqt, install_data_pyqt

# output directory for sip
if not exists('qth'):
    mkdir('qth')

if not exists('qth-lib'):
    mkdir('qth-lib')


#
# EXTENSION MODULE INFO
#

source_files = glob(join(srcdir, '*.cpp'))

PyQTH = PyQtExtension('QTiffHandler.qth',
                      source_files,
                      sip_spec_dirs = sip_spec_dirs,
                      moc_headers = ['%s/qtiffhandler.h' % srcdir],
                      include_dirs = ['%s' % srcdir],
                      libraries = ['tiff'])

#
# PACKAGE INFO
#

# examples
data_files = []

# pyuic examples
pyuic_files = []

pyuic_dir = 'pyuic%s' % QT_VERSION[0]
#data_files.append(('src/pyuic',
#                   map(lambda x: os.path.join(pyuic_dir, x), pyuic_files)))

# documentation
#data_files.append(glob('doc/*'))


long_description = """
PyQTH = Qt tiff handler for PyQT!
"""

setup(name              = "PyQTH",
      version           = version,
      licence           = "LGPL",
      url               = "http://lisa-gmbh.de",
      author            = "Hans-Peter Jansen",
      author_email      = "hpj@urpla.net",
      description       = "Python bindings for the QTiffHandler library",
      long_description  = long_description,
      cmdclass          = {'build_ext': build_ext_pyqt,
                           'install_data': install_data_pyqt},
      packages          = ['qth'],
      package_dir       = {'qth': 'qth-lib'},
      ext_modules       = [PyQTH],
      data_files        = data_files)

# Local Variables: ***
# compile-command: "python setup.py build" ***
# End: ***
