/***************************************************************************
  qtiffhandler.h    : Qt interface to fax tiff files
  ------------------------------------------------------------------
  copyright         : (C) 2002 by Serghei Amelian <serghei@argus.ro>
                    : (C) 2002 by Hans-Peter Jansen <hoj@urpla.net>
  changelog
  ------------------------------------------------------------------
  Thu Aug 29 2002 serghei started
  Mon Dec 16 2002 hp      smooth scaling
  Fri Dec 20 2002 hp      wrapped with sip
  Sat Dec 21 2002 hp      clean up, reorganizations, flicker fix
  Sun Dec 22 2002 hp      default ctor args, status msg handling
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef QTIFFHANDLER_H
#define QTIFFHANDLER_H

#include <qobject.h>
#include <qpixmap.h>
#include <qptrlist.h>

class QLabel;

typedef struct tiff TIFF;

class QTIFFHandler : public QObject {
Q_OBJECT
public:
    enum Resolution { Normal=2, Fine=1, Undefined=0 };
    QTIFFHandler(QLabel *status, QWidget *parent=0, const char *name=0, float zoom=1.0, bool smoothScale=0);
    ~QTIFFHandler();
    int load(const char *fileName);
    void close();
    void setSmoothScale(bool activate) { _smoothScale = activate; }
    bool smoothScale() { return _smoothScale; }
    void setZoom(float zoomFactor) { _zoom = zoomFactor; }
    float zoom() { return _zoom; }
    void setPage(int page);
    void scale(float zoomFactor);
    void rotate(double degrees);
    int page() { return _page; }
    int pages() { return _pages; }
    QString sender() { return _sender; }
    QString time() { return _time; }
    Resolution resolution() { return _resolution; }
    const char *filename() { return _filename.latin1(); }
    QPixmap *currentPage() { if(_currentPage.width()) return &_currentPage; else return 0; }
signals:
    void update();
private:
    void setStatus(QString msg, bool saveOld = false);
    void restoreStatus();
    void error(const char *strerr, const char *reason);
    int getPage(TIFF *tif, Resolution resolution);
    int _page, _pages;
    QString _sender, _time;
    Resolution _resolution;
    bool _smoothScale;
    float _zoom, _degrees;
    QPtrList<QPixmap> originalPixmaps;
    QPtrList<QImage> originalImages;
    QPixmap _currentPage;
    QString _filename;
    QLabel *status;
    QString oldStatus;
    float xResolution;
};

#endif
